@@ -1,11 +1,11 @@
 # This is the bcfg2 support for blastwave packages (pkg-get)
-'''This provides bcfg2 support for blastwave'''
-__revision__ = '$Revision: 5408 $'
+"""This provides bcfg2 support for blastwave"""
+__revision__ = '$Revision$'
 
 import Bcfg2.Client.Tools.SYSV, tempfile
 
 class Blast(Bcfg2.Client.Tools.SYSV.SYSV):
-    '''Support for Blastwave packages'''
+    """Support for Blastwave packages"""
     pkgtype = 'blast'
     pkgtool = ("/opt/csw/bin/pkg-get install %s", ("%s", ["bname"]))
     name = 'Blast'
@@ -29,5 +29,5 @@ class Blast(Bcfg2.Client.Tools.SYSV.SYSV):
     # Remove comes from Bcfg2.Client.Tools.SYSV
 
     def FindExtraPackages(self):
-        '''Pass through to null FindExtra call'''
+        """Pass through to null FindExtra call."""
         return []
