@@ -1,5 +1,5 @@
-'''All POSIX Type client support for Bcfg2'''
-__revision__ = '$Revision: 5514 $'
+"""All POSIX Type client support for Bcfg2."""
+__revision__ = '$Revision$'
 
 from datetime import datetime
 from stat import S_ISVTX, S_ISGID, S_ISUID, S_IXUSR, S_IWUSR, S_IRUSR, S_IXGRP
@@ -7,6 +7,7 @@ from stat import S_IWGRP, S_IRGRP, S_IXOTH, S_IWOTH, S_IROTH, ST_MODE, S_ISDIR
 from stat import S_IFREG, ST_UID, ST_GID, S_ISREG, S_IFDIR, S_ISLNK, ST_MTIME
 import binascii
 import difflib
+import errno
 import grp
 import logging
 import os
@@ -17,6 +18,7 @@ import string
 import time
 import Bcfg2.Client.Tools
 import Bcfg2.Options
+from Bcfg2.Client import XML
 
 log = logging.getLogger('posix')
 
@@ -27,7 +29,7 @@ device_map = {'block': stat.S_IFBLK,
 
 
 def calcPerms(initial, perms):
-    '''This compares ondisk permissions with specified ones'''
+    """This compares ondisk permissions with specified ones."""
     pdisp = [{1:S_ISVTX, 2:S_ISGID, 4:S_ISUID},
              {1:S_IXUSR, 2:S_IWUSR, 4:S_IRUSR},
              {1:S_IXGRP, 2:S_IWGRP, 4:S_IRGRP},
@@ -43,10 +45,10 @@ def calcPerms(initial, perms):
     return tempperms
 
 def normUid(entry):
-    '''
+    """
        This takes a user name or uid and
-       returns the corresponding uid or False
-    '''
+       returns the corresponding uid or False.
+    """
     try:
         try:
             return int(entry.get('owner'))
@@ -57,10 +59,10 @@ def normUid(entry):
         return False
 
 def normGid(entry):
-    '''
+    """
        This takes a group name or gid and
-       returns the corresponding gid or False
-    '''
+       returns the corresponding gid or False.
+    """
     try:
         try:
             return int(entry.get('group'))
@@ -74,7 +76,7 @@ text_chars = "".join([chr(y) for y in range(32, 127)] + list("\n\r\t\b"))
 notrans = string.maketrans("", "")
 
 def isString(strng):
-    '''Returns true if a string contains no binary chars'''
+    """Returns true if a string contains no binary chars."""
     if "\0" in strng:
         return False
 
@@ -84,15 +86,10 @@ def isString(strng):
     return len(strng.translate(notrans, text_chars)) == 0
 
 class POSIX(Bcfg2.Client.Tools.Tool):
-    '''POSIX File support code'''
+    """POSIX File support code."""
     name = 'POSIX'
     __handles__ = [('ConfigFile', None),
                    ('Directory', None),
-                   ('Path', 'ConfigFile'),
-                   ('Path', 'Directory'),
-                   ('Path', 'HardLink'),
-                   ('Path', 'Permissions'),
-                   ('Path', 'SymLink'),
                    ('Path', 'device'),
                    ('Path', 'directory'),
                    ('Path', 'file'),
@@ -117,7 +114,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
     max_copies = setup['max_copies']
 
     def canInstall(self, entry):
-        '''Check if entry is complete for installation'''
+        """Check if entry is complete for installation."""
         if Bcfg2.Client.Tools.Tool.canInstall(self, entry):
             if (entry.tag, entry.text, entry.get('empty', 'false')) == \
                ('ConfigFile', None, 'false'):
@@ -127,7 +124,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             return False
 
     def VerifySymLink(self, entry, _):
-        '''Verify SymLink Entry'''
+        """Verify SymLink Entry."""
         try:
             sloc = os.readlink(entry.get('name'))
             if sloc == entry.get('to'):
@@ -145,7 +142,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             return False
 
     def InstallSymLink(self, entry):
-        '''Install SymLink Entry'''
+        """Install SymLink entry."""
         self.logger.info("Installing Symlink %s" % (entry.get('name')))
         if os.path.lexists(entry.get('name')):
             try:
@@ -172,7 +169,11 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             return False
 
     def VerifyDirectory(self, entry, modlist):
-        '''Verify Directory Entry'''
+        """Verify Directory entry."""
+        if entry.get('perms') == None or \
+           entry.get('owner') == None or \
+           entry.get('group') == None:
+            return False
         while len(entry.get('perms', '')) < 4:
             entry.set('perms', '0' + entry.get('perms', ''))
         try:
@@ -204,7 +205,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
         pruneTrue = True
         ex_ents = []
         if entry.get('prune', 'false') == 'true' \
-               and entry.tag == 'Directory':
+               and (entry.tag == 'Directory' or entry.get('type') == 'directory'):
+               # FIXME: need to verify both old and new POSIX types
             try:
                 entries = ['/'.join([entry.get('name'), ent]) \
                            for ent in os.listdir(entry.get('name'))]
@@ -218,6 +220,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                     nqtext += "Directory %s contains extra entries:" % entry.get('name')
                     nqtext += ":".join(ex_ents)
                     entry.set('qtest', nqtext)
+                    [entry.append(XML.Element('Prune', path=x)) for x in ex_ents]
             except OSError:
                 ex_ents = []
                 pruneTrue = True
@@ -262,7 +265,13 @@ class POSIX(Bcfg2.Client.Tools.Tool):
         return pTrue and pruneTrue
 
     def InstallDirectory(self, entry):
-        '''Install Directory Entry'''
+        """Install Directory entry."""
+        if entry.get('perms') == None or \
+           entry.get('owner') == None or \
+           entry.get('group') == None:
+            self.logger.error('Entry %s not completely specified. '
+                              'Try running bcfg2-repo-validate.' % (entry.get('name')))
+            return False
         self.logger.info("Installing Directory %s" % (entry.get('name')))
         try:
             fmode = os.lstat(entry.get('name'))
@@ -314,7 +323,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                                   (entry.get('name')))
                 return False
         if entry.get('prune', 'false') == 'true' and entry.get("qtest"):
-            for pname in entry.get("qtest").split(":"):
+            for pent in entry.findall('Prune'):
+                pname = pent.get('path')
                 ulfailed = False
                 if os.path.isdir(pname):
                     self.logger.info("Not removing extra directory %s, please check and remove manually" % pname)
@@ -330,7 +340,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
         return self.InstallPermissions(entry)
 
     def VerifyhardLink(self, entry, _):
-        '''Verify HardLink Entry'''
+        """Verify HardLink entry."""
         try:
             if os.path.samefile(entry.get('name'), entry.get('to')):
                 return True
@@ -348,7 +358,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             return False
 
     def InstallhardLink(self, entry):
-        '''Install HardLink Entry'''
+        """Install HardLink entry."""
         self.logger.info("Installing Hardlink %s" % (entry.get('name')))
         if os.path.lexists(entry.get('name')):
             try:
@@ -374,11 +384,17 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             return False
 
     def VerifyPermissions(self, entry, _):
-        '''Verify Permissions entry'''
+        """Verify Permissions entry"""
         return self.VerifyDirectory(entry, _)
 
     def InstallPermissions(self, entry):
-        '''Install POSIX Permissions'''
+        """Install POSIX permissions"""
+        if entry.get('perms') == None or \
+           entry.get('owner') == None or \
+           entry.get('group') == None:
+            self.logger.error('Entry %s not completely specified. '
+                              'Try running bcfg2-repo-validate.' % (entry.get('name')))
+            return False
         try:
             os.chown(entry.get('name'), normUid(entry), normGid(entry))
             os.chmod(entry.get('name'), calcPerms(S_IFDIR, entry.get('perms')))
@@ -389,7 +405,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             return False
 
     def Verifydevice(self, entry, _):
-        '''Verify device entry'''
+        """Verify device entry."""
         try:
             # check for file existence
             filestat = os.stat(entry.get('name'))
@@ -432,7 +448,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             return False
 
     def Installdevice(self, entry):
-        '''Install device entries'''
+        """Install device entries."""
         try:
             # check for existing paths and remove them
             filestat = os.lstat(entry.get('name'))
@@ -465,7 +481,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 return False
 
     def Verifynonexistent(self, entry, _):
-        '''Verify nonexistent entry'''
+        """Verify nonexistent entry."""
         # return true if path does _not_ exist
         return not os.path.lexists(entry.get('name'))
 
@@ -500,7 +516,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 self.logger.error("Failed to read %s: %s" % (error.filename, error.strerror))
 
     def VerifyConfigFile(self, entry, _):
-        '''Install ConfigFile Entry'''
+        """Install ConfigFile entry."""
         # configfile verify is permissions check + content check
         permissionStatus = self.VerifyDirectory(entry, _)
         tbin = False
@@ -579,12 +595,12 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                     nqtext += '\nDiff took too long to compute, no printable diff'
             entry.set('qtext', nqtext)
         qtxt = entry.get('qtext', '')
-        qtxt += "\nInstall ConfigFile %s: (y/N): " % (entry.get('name'))
+        qtxt += "\nInstall %s %s: (y/N): " % (entry.tag, entry.get('name'))
         entry.set('qtext', qtxt)
         return contentStatus and permissionStatus
 
     def InstallConfigFile(self, entry):
-        '''Install ConfigFile Entry'''
+        """Install ConfigFile entry."""
         self.logger.info("Installing ConfigFile %s" % (entry.get('name')))
 
         parent = "/".join(entry.get('name').split('/')[:-1])
@@ -676,7 +692,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                     return False
             return True
         except (OSError, IOError), err:
-            if err.errno == 13:
+            if err.errno == errno.EACCES:
                 self.logger.info("Failed to open %s for writing" % (entry.get('name')))
             else:
                 print(err)
@@ -698,14 +714,6 @@ class POSIX(Bcfg2.Client.Tools.Tool):
         ret = getattr(self, 'InstallConfigFile')
         return ret(entry)
 
-    def Verifynonexistent(self, entry, _):
-        # FIXME: not implemented
-        return True
-
-    def Installnonexistent(self, entry):
-        # FIXME: not implemented
-        return True
-
     def Verifypermissions(self, entry, _):
         ret = getattr(self, 'VerifyPermissions')
         return ret(entry, _)
