@@ -1,6 +1,6 @@
-'''Bcfg2 Support for RPMS'''
+"""Bcfg2 Support for RPMS"""
 
-__revision__ = '$Revision: 5651 $'
+__revision__ = '$Revision$'
 
 import ConfigParser
 import os.path
@@ -15,7 +15,7 @@ except NameError:
     from sets import Set as set
 
 class RPMng(Bcfg2.Client.Tools.PkgTool):
-    '''Support for RPM packages'''
+    """Support for RPM packages."""
     name = 'RPMng'
 
     __execs__ = ['/bin/rpm', '/var/lib/rpm']
@@ -42,6 +42,10 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
     def __init__(self, logger, setup, config):
         Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config)
 
+        # create a global ignore list used when ignoring particular
+        # files during package verification
+        self.ignores = [entry.get('name') for struct in config for entry in struct \
+                        if entry.get('type') == 'ignore']
         self.instance_status = {}
         self.extra_instances = []
         self.modlists = {}
@@ -133,7 +137,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
 
     def RefreshPackages(self):
-        '''
+        """
             Creates self.installed{} which is a dict of installed packages.
 
             The dict items are lists of nevra dicts.  This loosely matches the
@@ -147,7 +151,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                                       {'name':'foo', 'epoch':None,
                                        'version':'1', 'release':2,
                                        'arch':'x86_64'} ]
-        '''
+        """
         self.installed = {}
         refresh_ts = rpmtools.rpmtransactionset()
         # Don't bother with signature checks at this stage. The GPG keys might
@@ -165,7 +169,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         del refresh_ts
 
     def VerifyPackage(self, entry, modlist, pinned_version=None):
-        '''
+        """
             Verify Package status for entry.
             Performs the following:
                 - Checks for the presence of required Package Instances.
@@ -189,7 +193,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                 extra_instances = [ <Package Element Object>, ..... ]
 
               Constructs the text prompts for interactive mode.
-        '''
+        """
         instances = [inst for inst in entry if inst.tag == 'Instance' or inst.tag == 'Package']
         if instances == []:
             # We have an old style no Instance entry. Convert it to new style.
@@ -219,7 +223,16 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         if entry.get('name') in self.installed:
             # There is at least one instance installed.
             if self.pkg_checks == 'true' and entry.get('pkg_checks', 'true') == 'true':
-                if entry.get('name') in self.installOnlyPkgs:
+                rpmTs = rpm.TransactionSet()
+                rpmHeader = None
+                for h in rpmTs.dbMatch(rpm.RPMTAG_NAME, entry.get('name')):
+                    if rpmHeader is None or rpm.versionCompare(h, rpmHeader) > 0:
+                        rpmHeader = h
+                rpmProvides = [ h['provides'] for h in \
+                            rpmTs.dbMatch(rpm.RPMTAG_NAME, entry.get('name')) ]
+                rpmIntersection = set(rpmHeader['provides']) & \
+                                  set(self.installOnlyPkgs)
+                if len(rpmIntersection) > 0:
                     # Packages that should only be installed or removed.
                     # e.g. kernels.
                     self.logger.debug("        Install only package.")
@@ -364,7 +377,9 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
                             # Check the rpm verify file results against the modlist
                             # and entry and per Instance Ignores.
-                            ignores = [ig.get('name') for ig in entry.findall('Ignore')] + [ig.get('name') for ig in inst.findall('Ignore')]
+                            ignores = [ig.get('name') for ig in entry.findall('Ignore')] + \
+                                      [ig.get('name') for ig in inst.findall('Ignore')] + \
+                                      self.ignores
                             for file_result in result.get('files', []):
                                 if file_result[-1] not in modlist + ignores:
                                     instance_fail = True
@@ -440,12 +455,13 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         return True
 
     def RemovePackages(self, packages):
-        '''
+        """
            Remove specified entries.
 
            packages is a list of Package Entries with Instances generated
            by FindExtraPackages().
-        '''
+        
+        """
         self.logger.debug('Running RPMng.RemovePackages()')
 
         pkgspec_list = []
@@ -511,12 +527,13 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         self.extra = self.FindExtraPackages()
 
     def FixInstance(self, instance, inst_status):
-        '''
+        """"
            Control if a reinstall of a package happens or not based on the
            results from RPMng.VerifyPackage().
 
            Return True to reinstall, False to not reintstall.
-        '''
+
+        """
         fix = False
 
         if inst_status.get('installed', False) == False:
@@ -569,7 +586,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         return fix
 
     def Install(self, packages, states):
-        '''
+        """
            Try and fix everything that RPMng.VerifyPackages() found wrong for
            each Package Entry.  This can result in individual RPMs being
            installed (for the first time), reinstalled, deleted, downgraded
@@ -585,7 +602,8 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
              of a package.
            - Each package will be added to self.modified[] if its states{}
              entry is set to True.
-        '''
+
+        """
         self.logger.info('Runing RPMng.Install()')
 
         install_only_pkgs = []
@@ -720,9 +738,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             self.modified.append(entry)
 
     def canInstall(self, entry):
-        '''
-            test if entry has enough information to be installed
-        '''
+        """Test if entry has enough information to be installed."""
         if not self.handlesEntry(entry):
             return False
 
@@ -794,7 +810,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         return True
 
     def canVerify(self, entry):
-        '''
+        """
             Test if entry has enough information to be verified.
 
             Three types of entries are checked.
@@ -804,7 +820,8 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
            Also the old style entries get modified after the first
            VerifyPackage() run, so there needs to be a second test.
-        '''
+
+        """
         if not self.handlesEntry(entry):
             return False
 
@@ -830,6 +847,10 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
                     self.logger.error("Incomplete information for entry %s:%s; cannot verify" \
                                       % (entry.tag, entry.get('name')))
                     return False
+            elif entry.tag == 'Path' and entry.get('type') == 'ignore':
+                # ignored Paths are only relevant during failed package
+                # verification
+                pass
             else:
                 if [attr for attr in self.__req__[entry.tag] if attr not in entry.attrib]:
                     self.logger.error("Incomplete information for entry %s:%s; cannot verify" \
@@ -869,9 +890,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         return True
 
     def FindExtraPackages(self):
-        '''
-           Find extra packages
-        '''
+        """Find extra packages."""
         packages = [entry.get('name') for entry in self.getSupportedEntries()]
         extras = []
 
@@ -894,11 +913,12 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
 
     def FindExtraInstances(self, pkg_entry, installed_entry):
-        '''
+        """
             Check for installed instances that are not in the config.
             Return a Package Entry with Instances to remove, or None if there
             are no Instances to remove.
-        '''
+
+        """
         name = pkg_entry.get('name')
         extra_entry = Bcfg2.Client.XML.Element('Package', name=name, type=self.pkgtype)
         instances = [inst for inst in pkg_entry if inst.tag == 'Instance' or inst.tag == 'Package']
@@ -947,9 +967,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         return extra_entry
 
     def str_evra(self, instance):
-        '''
-            Convert evra dict entries to a string.
-        '''
+        """Convert evra dict entries to a string."""
         if instance.get('epoch', '*') in ['*', None]:
             return '%s-%s.%s' % (instance.get('version', '*'),
                                  instance.get('release', '*'),
@@ -973,9 +991,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             return False
 
     def inst_evra_equal(self, config_entry, installed_entry):
-        '''
-            Compare new style instance to installed entry.
-        '''
+        """Compare new style instance to installed entry."""
 
         if config_entry.get('epoch', None) != None:
             epoch = int(config_entry.get('epoch'))
@@ -994,12 +1010,13 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             return False
 
     def getinstalledgpg(self):
-        '''
+        """
            Create a list of installed GPG key IDs.
 
            The pgp-pubkey package version is the least significant 4 bytes
            (big-endian) of the key ID which is good enough for our purposes.
-        '''
+
+        """
         init_ts = rpmtools.rpmtransactionset()
         init_ts.setVSFlags(rpm._RPMVSF_NODIGESTS|rpm._RPMVSF_NOSIGNATURES)
         gpg_hdrs = rpmtools.getheadersbykeyword(init_ts, **{'name':'gpg-pubkey'})
@@ -1008,3 +1025,10 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         init_ts.closeDB()
         del init_ts
         return keyids
+
+    def VerifyPath(self, entry, _):
+        """
+           We don't do anything here since all
+           Paths are processed in __init__
+        """
+        return True
