@@ -1,13 +1,4 @@
-'''This contains all Bcfg2 Tool modules'''
-__revision__ = '$Revision: 5445 $'
-
-__all__ = ["Action", "APT", "Blast", "Chkconfig", "DebInit", "Encap", "IPS",
-           "FreeBSDInit", "FreeBSDPackage", "launchd", "Portage", "POSIX",
-           "RPMng", 'rpmtools', "RcUpdate", "SMF", "SYSV", "YUMng"]
-
-drivers = [item for item in __all__ if item not in ['rpmtools']]
-default = [item for item in drivers if item not in ['RPM', 'Yum']]
-
+"""This contains all Bcfg2 Tool modules"""
 # suppress popen2 warnings for python 2.3
 import warnings
 warnings.filterwarnings("ignore", "The popen2 module is deprecated.*",
@@ -19,13 +10,24 @@ import sys
 import time
 
 import Bcfg2.Client.XML
+__revision__ = '$Revision$'
+
+__all__ = [tool.split('.')[0] \
+           for tool in os.listdir(os.path.dirname(__file__)) \
+           if tool.endswith(".py") and tool != "__init__.py"]
+
+drivers = [item for item in __all__ if item not in ['rpmtools']]
+default = [item for item in drivers if item not in ['RPM', 'Yum']]
+
 
 class toolInstantiationError(Exception):
-    '''This error is called if the toolset cannot be instantiated'''
+    """This error is called if the toolset cannot be instantiated."""
     pass
 
+
 class readonlypipe(popen2.Popen4):
-    '''This pipe sets up stdin --> /dev/null'''
+    """This pipe sets up stdin --> /dev/null."""
+
     def __init__(self, cmd, bufsize=-1):
         popen2._cleanup()
         c2pread, c2pwrite = os.pipe()
@@ -42,13 +44,15 @@ class readonlypipe(popen2.Popen4):
         self.fromchild = os.fdopen(c2pread, 'r', bufsize)
         popen2._active.append(self)
 
+
 class executor:
-    '''this class runs stuff for us'''
+    """This class runs stuff for us"""
+
     def __init__(self, logger):
         self.logger = logger
 
     def run(self, command):
-        '''Run a command in a pipe dealing with stdout buffer overloads'''
+        """Run a command in a pipe dealing with stdout buffer overloads."""
         self.logger.debug('> %s' % command)
 
         runpipe = readonlypipe(command, bufsize=16384)
@@ -69,12 +73,16 @@ class executor:
             cmdstat = runpipe.poll()
         output += [line[:-1] for line in runpipe.fromchild.readlines() \
                    if line]
-        return (cmdstat, output)
+        # The exit code from the program is in the upper byte of the
+        # value returned by cmdstat. Shift it down for tools looking at
+        # the value.
+        return ((cmdstat >> 8), output)
+
 
 class Tool:
-    '''
-    All tools subclass this. It defines all interfaces that need to be defined
-    '''
+    """
+    All tools subclass this. It defines all interfaces that need to be defined.
+    """
     name = 'Tool'
     __execs__ = []
     __handles__ = []
@@ -82,6 +90,10 @@ class Tool:
     __important__ = []
 
     def __init__(self, logger, setup, config):
+        self.__important__ = [entry.get('name') \
+                              for struct in config for entry in struct \
+                              if entry.tag == 'Path' and \
+                              entry.get('important') in ['true', 'True']]
         self.setup = setup
         self.logger = logger
         if not hasattr(self, '__ireq__'):
@@ -106,15 +118,15 @@ class Tool:
                 raise toolInstantiationError
 
     def BundleUpdated(self, _, states):
-        '''This callback is used when bundle updates occur'''
+        """This callback is used when bundle updates occur."""
         return
 
     def BundleNotUpdated(self, _, states):
-        '''This callback is used when a bundle is not updated'''
+        """This callback is used when a bundle is not updated."""
         return
 
     def Inventory(self, states, structures=[]):
-        '''Dispatch verify calls to underlying methods'''
+        """Dispatch verify calls to underlying methods."""
         if not structures:
             structures = self.config.getchildren()
         mods = self.buildModlist()
@@ -131,7 +143,7 @@ class Tool:
         self.extra = self.FindExtra()
 
     def Install(self, entries, states):
-        '''Install all entries in sublist'''
+        """Install all entries in sublist."""
         for entry in entries:
             try:
                 func = getattr(self, "Install%s" % (entry.tag))
@@ -142,18 +154,18 @@ class Tool:
                                   % (entry.tag), exc_info=1)
 
     def Remove(self, entries):
-        '''Remove specified extra entries'''
+        """Remove specified extra entries"""
         pass
 
     def getSupportedEntries(self):
-        '''return a list of supported entries'''
+        """Return a list of supported entries."""
         return [entry for struct in \
                 self.config.getchildren() for entry in \
                 struct.getchildren() \
                 if self.handlesEntry(entry)]
 
     def handlesEntry(self, entry):
-        '''return if entry is handled by this Tool'''
+        """Return if entry is handled by this tool."""
         return (entry.tag, entry.get('type')) in self.__handles__
 
     def buildModlist(self):
@@ -164,11 +176,11 @@ class Tool:
                                  'Permissions', 'Ignore', 'Path']]
 
     def gatherCurrentData(self, entry):
-        '''Default implementation of the information gathering routines'''
+        """Default implementation of the information gathering routines."""
         pass
 
     def canVerify(self, entry):
-        '''test if entry has enough information to be verified'''
+        """Test if entry has enough information to be verified."""
         if not self.handlesEntry(entry):
             return False
 
@@ -191,13 +203,12 @@ class Tool:
             return False
         return True
 
-
     def FindExtra(self):
-        '''Return a list of extra entries'''
+        """Return a list of extra entries."""
         return []
 
     def canInstall(self, entry):
-        '''test if entry has enough information to be installed'''
+        """Test if entry has enough information to be installed."""
         if not self.handlesEntry(entry):
             return False
 
@@ -216,11 +227,12 @@ class Tool:
             return False
         return True
 
+
 class PkgTool(Tool):
-    '''
+    """
        PkgTool provides a one-pass install with
        fallback for use with packaging systems
-    '''
+    """
     pkgtool = ('echo %s', ('%s', ['name']))
     pkgtype = 'echo'
     name = 'PkgTool'
@@ -233,14 +245,14 @@ class PkgTool(Tool):
         self.RefreshPackages()
 
     def VerifyPackage(self, dummy, _):
-        '''Dummy verification method'''
+        """Dummy verification method"""
         return False
 
     def Install(self, packages, states):
-        '''
+        """
            Run a one-pass install, followed by
-           single pkg installs in case of failure
-        '''
+           single pkg installs in case of failure.
+        """
         self.logger.info("Trying single pass package install for pkgtype %s" % \
                          self.pkgtype)
 
@@ -259,7 +271,8 @@ class PkgTool(Tool):
                           if entry.tag == 'Package'
                           and entry.get('type') == self.pkgtype
                           and entry.get('name') in pkgnames]:
-                self.logger.debug('Setting state to true for pkg %s' % (entry.get('name')))
+                self.logger.debug('Setting state to true for pkg %s' % \
+                                  (entry.get('name')))
                 states[entry] = True
             self.RefreshPackages()
         else:
@@ -269,7 +282,8 @@ class PkgTool(Tool):
             for pkg in packages:
                 # handle state tracking updates
                 if self.VerifyPackage(pkg, []):
-                    self.logger.info("Forcing state to true for pkg %s" % (pkg.get('name')))
+                    self.logger.info("Forcing state to true for pkg %s" % \
+                                     (pkg.get('name')))
                     states[pkg] = True
                 else:
                     self.logger.info("Installing pkg %s version %s" %
@@ -280,21 +294,22 @@ class PkgTool(Tool):
                     if cmdrc[0] == 0:
                         states[pkg] = True
                     else:
-                        self.logger.error("Failed to install package %s" % (pkg.get('name')))
+                        self.logger.error("Failed to install package %s" % \
+                                          (pkg.get('name')))
             self.RefreshPackages()
         for entry in [ent for ent in packages if states[ent]]:
             self.modified.append(entry)
 
     def RefreshPackages(self):
-        '''Dummy state refresh method'''
+        """Dummy state refresh method."""
         pass
 
     def RemovePackages(self, packages):
-        '''Dummy implementation of package removal method'''
+        """Dummy implementation of package removal method."""
         pass
 
     def FindExtraPackages(self):
-        '''Find extra packages'''
+        """Find extra packages."""
         packages = [entry.get('name') for entry in self.getSupportedEntries()]
         extras = [data for data in list(self.installed.items()) \
                   if data[0] not in packages]
@@ -302,12 +317,13 @@ class PkgTool(Tool):
                                          type=self.pkgtype, version=version) \
                                          for (name, version) in extras]
 
+
 class SvcTool(Tool):
-    '''This class defines basic Service behavior'''
+    """This class defines basic Service behavior"""
     name = 'SvcTool'
 
     def get_svc_command(self, service, action):
-        '''Return the basename of the command used to start/stop services'''
+        """Return the basename of the command used to start/stop services."""
         return '/etc/init.d/%s %s' % (service.get('name'), action)
 
     def start_service(self, service):
@@ -330,7 +346,7 @@ class SvcTool(Tool):
         return 0
 
     def BundleUpdated(self, bundle, states):
-        '''The Bundle has been updated'''
+        """The Bundle has been updated."""
         if self.setup['servicemode'] == 'disabled':
             return
 
