@@ -1,26 +1,29 @@
 import getopt
+import sys
 import Bcfg2.Server.Admin
 
 class Pull(Bcfg2.Server.Admin.MetadataCore):
-    '''
-    Pull mode retrieves entries from clients and
-    integrates the information into the repository
-    '''
+    """Pull mode retrieves entries from clients and
+    integrates the information into the repository.
+    """
     __shorthelp__ = ("Integrate configuration information "
                      "from clients into the server repository")
-    __longhelp__ = (__shorthelp__ + "\n\nbcfg2-admin pull [-v] [-f][-I] "
+    __longhelp__ = (__shorthelp__ + "\n\nbcfg2-admin pull [-v] [-f][-I] [-s] "
                                     "<client> <entry type> <entry name>")
     __usage__ = ("bcfg2-admin pull [options] <client> <entry type> "
                  "<entry name>\n\n"
                  "     %-25s%s\n"
                  "     %-25s%s\n"
+                 "     %-25s%s\n"
                  "     %-25s%s\n" %
                 ("-v",
                  "be verbose",
                  "-f",
                  "force",
                  "-I",
-                 "interactive"))
+                 "interactive",
+                 "-s",
+                 "stdin"))
     allowed = ['Metadata', 'BB', "DBStats", "Statistics", "Cfg", "SSHbase"]
 
     def __init__(self, configfile):
@@ -31,8 +34,9 @@ class Pull(Bcfg2.Server.Admin.MetadataCore):
 
     def __call__(self, args):
         Bcfg2.Server.Admin.Mode.__call__(self, args)
+        use_stdin = False
         try:
-            opts, gargs = getopt.getopt(args, 'vfI')
+            opts, gargs = getopt.getopt(args, 'vfIs')
         except:
             print self.__shorthelp__
             raise SystemExit(1)
@@ -43,10 +47,25 @@ class Pull(Bcfg2.Server.Admin.MetadataCore):
                 self.mode = 'force'
             elif opt[0] == '-I':
                 self.mode == 'interactive'
-        self.PullEntry(gargs[0], gargs[1], gargs[2])
+            elif opt[0] == '-s':
+                use_stdin = True
+
+        if use_stdin:
+            for line in sys.stdin:
+                try:
+                    self.PullEntry(*line.split(None, 3))
+                except SystemExit:
+                    print "  for %s" % line
+                except:
+                    print "Bad entry: %s" % line.strip()
+        elif len(gargs) < 3:
+            print self.__longhelp__
+            raise SystemExit(1)
+        else:
+            self.PullEntry(gargs[0], gargs[1], gargs[2])
 
     def BuildNewEntry(self, client, etype, ename):
-        '''construct a new full entry for given client/entry from statistics'''
+        """Construct a new full entry for given client/entry from statistics."""
         new_entry = {'type':etype, 'name':ename}
         for plugin in self.bcore.pull_sources:
             try:
@@ -71,7 +90,7 @@ class Pull(Bcfg2.Server.Admin.MetadataCore):
         return new_entry
 
     def Choose(self, choices):
-        '''Determine where to put pull data'''
+        """Determine where to put pull data."""
         if self.mode == 'interactive':
             for choice in choices:
                 print "Plugin returned choice:"
@@ -94,7 +113,7 @@ class Pull(Bcfg2.Server.Admin.MetadataCore):
             return choices[0]
 
     def PullEntry(self, client, etype, ename):
-        '''Make currently recorded client state correct for entry'''
+        """Make currently recorded client state correct for entry."""
         new_entry = self.BuildNewEntry(client, etype, ename)
 
         meta = self.bcore.build_metadata(client)
