@@ -1,5 +1,5 @@
-'''Bcfg2.Server.Core provides the runtime support for bcfg2 modules'''
-__revision__ = '$Revision: 5650 $'
+"""Bcfg2.Server.Core provides the runtime support for Bcfg2 modules."""
+__revision__ = '$Revision$'
 
 import atexit
 import logging
@@ -17,7 +17,7 @@ import Bcfg2.Server.Plugins.Metadata
 logger = logging.getLogger('Bcfg2.Server.Core')
 
 def critical_error(operation):
-    '''Log and err, traceback and return an xmlrpc fault to client'''
+    """Log and err, traceback and return an xmlrpc fault to client."""
     logger.error(operation, exc_info=1)
     raise xmlrpclib.Fault(7, "Critical unexpected failure: %s" % (operation))
 
@@ -28,15 +28,16 @@ except:
     pass
 
 class CoreInitError(Exception):
-    '''This error is raised when the core cannot be initialized'''
+    """This error is raised when the core cannot be initialized."""
     pass
 
 class Core(Component):
-    '''The Core object is the container for all Bcfg2 Server logic, and modules'''
+    """The Core object is the container for all Bcfg2 Server logic and modules."""
     name = 'bcfg2-server'
     implementation = 'bcfg2-server'
 
-    def __init__(self, repo, plugins, password, encoding, ca=None,
+    def __init__(self, repo, plugins, password, encoding,
+                 cfile='/etc/bcfg2.conf', ca=None,
                  filemonitor='default', start_fam_thread=False):
         Component.__init__(self)
         self.datastore = repo
@@ -51,13 +52,16 @@ class Core(Component):
             raise CoreInitError, "failed to instantiate fam driver (used %s)" % \
                   filemonitor
         self.pubspace = {}
+        self.cfile = cfile
         self.cron = {}
         self.plugins = {}
+        self.plugin_blacklist = {}
         self.revision = '-1'
         self.password = password
         self.encoding = encoding
-        self.shutdown_flag = False
         atexit.register(self.shutdown)
+        # Create an event to signal worker threads to shutdown
+        self.terminate = threading.Event()
 
         if '' in plugins:
             plugins.remove('')
@@ -65,6 +69,14 @@ class Core(Component):
         for plugin in plugins:
             if not plugin in self.plugins:
                 self.init_plugins(plugin)
+        # Remove blacklisted plugins
+        for p, bl in self.plugin_blacklist.items():
+            if len(bl) > 0:
+                logger.error("The following plugins conflict with %s;"
+                             "Unloading %s" % (p, bl))
+            for plug in bl:
+                del self.plugins[plug]
+        # This section loads the experimental plugins
         expl = [plug for (name, plug) in self.plugins.iteritems()
                 if plug.experimental]
         if expl:
@@ -73,6 +85,7 @@ class Core(Component):
             logger.info("NOTE: Interfaces subject to change")
         depr = [plug for (name, plug) in self.plugins.iteritems()
                 if plug.deprecated]
+        # This section loads the deprecated plugins
         if depr:
             logger.info("Loading deprecated plugin(s): %s" % \
                         (" ".join([x.name for x in depr])))
@@ -101,14 +114,16 @@ class Core(Component):
             self.fam_thread.start()
 
     def _file_monitor_thread(self):
+        """The thread for monitor the files."""
         famfd = self.fam.fileno()
-        while not self.shutdown_flag:
+        terminate = self.terminate
+        while not terminate.isSet():
             try:
                 if famfd:
-                    select.select([famfd], [], [])
+                    select.select([famfd], [], [], 2)
                 else:
-                    while not self.fam.pending():
-                        time.sleep(15)
+                    if not self.fam.pending():
+                        terminate.wait(15)
                 self.fam.handle_event_set(self.lock)
             except:
                 continue
@@ -118,6 +133,7 @@ class Core(Component):
                     self.revision = plugin.get_revision()
 
     def init_plugins(self, plugin):
+        """Handling for the plugins."""
         try:
             mod = getattr(__import__("Bcfg2.Server.Plugins.%s" %
                                 (plugin)).Server.Plugins, plugin)
@@ -128,6 +144,10 @@ class Core(Component):
                 logger.error("Failed to load plugin %s" % (plugin))
                 return
         plug = getattr(mod, plugin)
+        # Blacklist conflicting plugins
+        cplugs = [conflict for conflict in plug.conflicts
+                  if conflict in self.plugins]
+        self.plugin_blacklist[plug.name] = cplugs
         try:
             self.plugins[plugin] = plug(self, self.datastore)
         except PluginInitError:
@@ -137,11 +157,14 @@ class Core(Component):
                 (plugin), exc_info=1)
 
     def shutdown(self):
-        self.shutdown_flag = True
-        for plugin in self.plugins.values():
-            plugin.shutdown()
+        """Shuting down the plugins."""
+        if not self.terminate.isSet():
+            self.terminate.set()
+            for plugin in self.plugins.values():
+                plugin.shutdown()
 
     def validate_data(self, metadata, data, base_cls):
+        """Checks the data structure."""
         for plugin in self.plugins.values():
             if isinstance(plugin, base_cls):
                 try:
@@ -154,11 +177,11 @@ class Core(Component):
                                  % (plugin.name, err.message))
                     raise
                 except:
-                    logger.error("Plugin %s: unexpected structure val failure" \
+                    logger.error("Plugin %s: unexpected structure validation failure" \
                                  % (plugin.name), exc_info=1)
 
     def GetStructures(self, metadata):
-        '''Get all structures for client specified by metadata'''
+        """Get all structures for client specified by metadata."""
         structures = reduce(lambda x, y:x+y,
                             [struct.BuildStructures(metadata) for struct \
                              in self.structures], [])
@@ -170,7 +193,7 @@ class Core(Component):
         return structures
 
     def BindStructure(self, structure, metadata):
-        '''Bind a complete structure'''
+        """Bind a complete structure."""
         for entry in structure.getchildren():
             if entry.tag.startswith("Bound"):
                 entry.tag = entry.tag[5:]
@@ -187,7 +210,7 @@ class Core(Component):
                              % (entry.tag, entry.get('name')), exc_info=1)
 
     def Bind(self, entry, metadata):
-        '''Bind an entry using the appropriate generator'''
+        """Bind an entry using the appropriate generator."""
         if 'altsrc' in entry.attrib:
             oldname = entry.get('name')
             entry.set('name', entry.get('altsrc'))
@@ -222,7 +245,7 @@ class Core(Component):
         raise PluginExecutionError, (entry.tag, entry.get('name'))
 
     def BuildConfiguration(self, client):
-        '''Build Configuration for client'''
+        """Build configuration for clients."""
         start = time.time()
         config = lxml.etree.Element("Configuration", version='2.0', \
                                     revision=self.revision)
@@ -265,6 +288,7 @@ class Core(Component):
         return config
 
     def GetDecisions(self, metadata, mode):
+        """Get data for the decision list."""
         result = []
         for plugin in self.plugins.values():
             try:
@@ -276,6 +300,10 @@ class Core(Component):
         return result
 
     def build_metadata(self, client_name):
+        """Build the metadata structure."""		
+        if not hasattr(self, 'metadata'):
+            # some threads start before metadata is even loaded
+            raise Bcfg2.Server.Plugins.Metadata.MetadataRuntimeError
         imd = self.metadata.get_initial_metadata(client_name)
         for conn in self.connectors:
             grps = conn.get_additional_groups(imd)
@@ -287,6 +315,7 @@ class Core(Component):
         return imd
 
     def process_statistics(self, client_name, statistics):
+        """Proceed statistics for client."""
         meta = self.build_metadata(client_name)
         state = statistics.find(".//Statistics")
         if state.get('version') >= '2.0':
@@ -304,7 +333,7 @@ class Core(Component):
 
     @exposed
     def GetProbes(self, address):
-        '''Fetch probes for a particular client'''
+        """Fetch probes for a particular client."""
         resp = lxml.etree.Element('probes')
         try:
             name = self.metadata.resolve_client(address)
@@ -320,17 +349,21 @@ class Core(Component):
             warning = 'Client metadata resolution error for %s; check server log' % address[0]
             self.logger.warning(warning)
             raise xmlrpclib.Fault(6, warning)
+        except Bcfg2.Server.Plugins.Metadata.MetadataRuntimeError:
+            err_msg = 'Metadata system runtime failure'
+            self.logger.error(err_msg)
+            raise xmlrpclib.Fault(6, err_msg)
         except:
-            critical_error("error determining client probes")
+            critical_error("Error determining client probes")
 
     @exposed
     def RecvProbeData(self, address, probedata):
-        '''Receive probe data from clients'''
+        """Receive probe data from clients."""
         try:
             name = self.metadata.resolve_client(address)
             meta = self.build_metadata(name)
         except Bcfg2.Server.Plugins.Metadata.MetadataConsistencyError:
-            warning = 'metadata consistency error'
+            warning = 'Metadata consistency error'
             self.logger.warning(warning)
             raise xmlrpclib.Fault(6, warning)
         # clear dynamic groups
@@ -359,20 +392,20 @@ class Core(Component):
 
     @exposed
     def AssertProfile(self, address, profile):
-        '''Set profile for a client'''
+        """Set profile for a client."""
         try:
             client = self.metadata.resolve_client(address)
             self.metadata.set_profile(client, profile, address)
         except (Bcfg2.Server.Plugins.Metadata.MetadataConsistencyError,
                 Bcfg2.Server.Plugins.Metadata.MetadataRuntimeError):
-            warning = 'metadata consistency error'
+            warning = 'Metadata consistency error'
             self.logger.warning(warning)
             raise xmlrpclib.Fault(6, warning)
         return True
 
     @exposed
     def GetConfig(self, address, checksum=False):
-        '''Build config for a client'''
+        """Build config for a client."""
         try:
             client = self.metadata.resolve_client(address)
             config = self.BuildConfiguration(client)
@@ -384,7 +417,7 @@ class Core(Component):
 
     @exposed
     def RecvStats(self, address, stats):
-        '''Act on statistics upload'''
+        """Act on statistics upload."""
         sdata = lxml.etree.XML(stats)
         client = self.metadata.resolve_client(address)
         self.process_statistics(client, sdata)
@@ -394,12 +427,13 @@ class Core(Component):
         if self.ca:
             acert = cert
         else:
-            # no ca, so no cert validation can be done
+            # No ca, so no cert validation can be done
             acert = None
         return self.metadata.AuthenticateConnection(acert, user, password, address)
 
     @exposed
     def GetDecisionList(self, address, mode):
+        """Get the data of the decision list."""
         client = self.metadata.resolve_client(address)
         meta = self.build_metadata(client)
         return self.GetDecisions(meta, mode)
