@@ -1,22 +1,32 @@
+import binascii
+import difflib
+import logging
+import lxml.etree
+import platform
+import time
+
+try:
+    from django.core.exceptions import MultipleObjectsReturned
+except ImportError:
+    pass
+
 import Bcfg2.Server.Plugin
 import Bcfg2.Server.Reports.importscript
 from Bcfg2.Server.Reports.reports.models import Client
-import difflib, lxml.etree, time, logging
 import Bcfg2.Server.Reports.settings
-
 from Bcfg2.Server.Reports.updatefix import update_database
 # for debugging output only
 logger = logging.getLogger('Bcfg2.Plugins.DBStats')
 
 class DBStats(Bcfg2.Server.Plugin.Plugin,
-              Bcfg2.Server.Plugin.Statistics,
+              Bcfg2.Server.Plugin.ThreadedStatistics,
               Bcfg2.Server.Plugin.PullSource):
     name = 'DBStats'
-    __version__ = '$Id: DBStats.py 5530 2009-11-05 05:23:34Z desai $'
+    __version__ = '$Id$'
 
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
-        Bcfg2.Server.Plugin.Statistics.__init__(self)
+        Bcfg2.Server.Plugin.ThreadedStatistics.__init__(self, core, datastore)
         Bcfg2.Server.Plugin.PullSource.__init__(self)
         self.cpath = "%s/Metadata/clients.xml" % datastore
         self.core = core
@@ -27,33 +37,40 @@ class DBStats(Bcfg2.Server.Plugin.Plugin,
             logger.debug(str(inst))
             logger.debug(str(type(inst)))
 
-    def process_statistics(self, mdata, xdata):
-        newstats = xdata.find("Statistics")
+    def handle_statistic(self, metadata, data):
+        newstats = data.find("Statistics")
         newstats.set('time', time.asctime(time.localtime()))
         # ick
-        xdata = lxml.etree.tostring(newstats)
-        ndx = lxml.etree.XML(xdata)
-        e = lxml.etree.Element('Node', name=mdata.hostname)
+        data = lxml.etree.tostring(newstats)
+        ndx = lxml.etree.XML(data)
+        e = lxml.etree.Element('Node', name=metadata.hostname)
         e.append(ndx)
         container = lxml.etree.Element("ConfigStatistics")
         container.append(e)
 
         # FIXME need to build a metadata interface to expose a list of clients
-        # FIXME Server processing the request should be mentionned here
         start = time.time()
         for i in [1, 2, 3]:
             try:
-                Bcfg2.Server.Reports.importscript.load_stats(
-                    self.core.metadata.clientdata, container, 0, True)
+                Bcfg2.Server.Reports.importscript.load_stats(self.core.metadata.clientdata,
+                                                             container,
+                                                             0,
+                                                             logger,
+                                                             True,
+                                                             platform.node())
+                logger.info("Imported data for %s in %s seconds" \
+                            % (metadata.hostname, time.time() - start))
+                return
+            except MultipleObjectsReturned, e:
+                logger.error("DBStats: MultipleObjectsReturned while handling %s: %s" % \
+                    (metadata.hostname, e))
+                logger.error("DBStats: Data is inconsistent")
                 break
             except:
                 logger.error("DBStats: Failed to write to db (lock); retrying",
                              exc_info=1)
-            if i == 3:
-                logger.error("DBStats: Retry limit failed; aborting operation")
-                return
-        logger.info("Imported data in the reason fast path in %s second" \
-                    % (time.time() - start))
+        logger.error("DBStats: Retry limit failed for %s; aborting operation" \
+                    % metadata.hostname)
 
     def GetExtra(self, client):
         c_inst = Client.objects.filter(name=client)[0]
@@ -61,7 +78,11 @@ class DBStats(Bcfg2.Server.Plugin.Plugin,
                 c_inst.current_interaction.extra()]
 
     def GetCurrentEntry(self, client, e_type, e_name):
-        c_inst = Client.objects.filter(name=client)[0]
+        try:
+            c_inst = Client.objects.filter(name=client)[0]
+        except IndexError:
+            self.logger.error("Unknown client: %s" % client)
+            raise Bcfg2.Server.Plugin.PluginExecutionError
         result = c_inst.current_interaction.bad().filter(entry__kind=e_type,
                                                          entry__name=e_name)
         if not result:
@@ -76,8 +97,14 @@ class DBStats(Bcfg2.Server.Plugin.Plugin,
                 ret.append(getattr(entry.reason, "current_%s" % t))
 
         if entry.reason.current_diff != '':
-            ret.append('\n'.join(difflib.restore(\
-                entry.reason.current_diff.split('\n'), 1)))
+            if entry.reason.is_binary:
+                ret.append(binascii.a2b_base64(entry.reason.current_diff))
+            else:
+                ret.append('\n'.join(difflib.restore(\
+                    entry.reason.current_diff.split('\n'), 1)))
+        elif entry.reason.is_binary:
+            # If len is zero the object was too large to store
+            raise Bcfg2.Server.Plugin.PluginExecutionError
         else:
             ret.append(None)
         return ret
