@@ -3,7 +3,7 @@ import re
 import lxml.etree
 
 def linesub(pattern, repl, filestring):
-    '''Substitutes instances of pattern with repl in filestring'''
+    """Substitutes instances of pattern with repl in filestring."""
     if filestring == None:
         filestring = ''
     output = list()
@@ -13,9 +13,9 @@ def linesub(pattern, repl, filestring):
     return '\n'.join(output)
 
 class EditDirectives(Bcfg2.Server.Plugin.SpecificData):
-    '''This object handles the editing directives'''
+    """This object handles the editing directives."""
     def ProcessDirectives(self, input):
-        '''Processes a list of edit directives on input'''
+        """Processes a list of edit directives on input."""
         temp = input
         for directive in self.data.split('\n'):
             directive = directive.split(',')
@@ -51,7 +51,7 @@ class EditEntrySet(Bcfg2.Server.Plugin.EntrySet):
 class Editor(Bcfg2.Server.Plugin.GroupSpool,
              Bcfg2.Server.Plugin.Probing):
     name = 'Editor'
-    __version__ = '$Id: Editor.py 5477 2009-10-07 14:46:08Z solj $'
+    __version__ = '$Id$'
     __author__ = 'bcfg2-dev@mcs.anl.gov'
     filename_pattern = 'edits'
     es_child_cls = EditDirectives
