@@ -1,24 +1,26 @@
-'''This file stores persistent metadata for the Bcfg2 Configuration Repository'''
-__revision__ = '$Revision: 5673 $'
+"""This file stores persistent metadata for the Bcfg2 Configuration Repository."""
+
+__revision__ = '$Revision$'
 
 import copy
 import fcntl
 import lxml.etree
 import os
+import os.path
 import socket
 import time
 import Bcfg2.Server.Plugin
 
 class MetadataConsistencyError(Exception):
-    '''This error gets raised when metadata is internally inconsistent'''
+    """This error gets raised when metadata is internally inconsistent."""
     pass
 
 class MetadataRuntimeError(Exception):
-    '''This error is raised when the metadata engine is called prior to reading enough data'''
+    """This error is raised when the metadata engine is called prior to reading enough data."""
     pass
 
 class ClientMetadata(object):
-    '''This object contains client metadata'''
+    """This object contains client metadata."""
     def __init__(self, client, profile, groups, bundles,
                  aliases, addresses, categories, uuid, password, query):
         self.hostname = client
@@ -34,17 +36,24 @@ class ClientMetadata(object):
         self.query = query
 
     def inGroup(self, group):
-        '''Test to see if client is a member of group'''
+        """Test to see if client is a member of group."""
         return group in self.groups
 
+    def group_in_category(self, category):
+        for grp in self.query.all_groups_in_category(category):
+            if grp in self.groups:
+                return grp
+        return ''
+
 class MetadataQuery(object):
-    def __init__(self, by_name, get_clients, by_groups, by_profiles, all_groups):
+    def __init__(self, by_name, get_clients, by_groups, by_profiles, all_groups, all_groups_in_category):
         # resolver is set later
         self.by_name = by_name
         self.names_by_groups = by_groups
         self.names_by_profiles = by_profiles
         self.all_clients = get_clients
         self.all_groups = all_groups
+        self.all_groups_in_category = all_groups_in_category
 
     def by_groups(self, groups):
         return [self.by_name(name) for name in self.names_by_groups(groups)]
@@ -58,8 +67,8 @@ class MetadataQuery(object):
 class Metadata(Bcfg2.Server.Plugin.Plugin,
                Bcfg2.Server.Plugin.Metadata,
                Bcfg2.Server.Plugin.Statistics):
-    '''This class contains data for bcfg2 server metadata'''
-    __version__ = '$Id: Metadata.py 5673 2010-01-13 01:54:54Z solj $'
+    """This class contains data for bcfg2 server metadata."""
+    __version__ = '$Id$'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     name = "Metadata"
 
@@ -72,6 +81,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
                 core.fam.AddMonitor("%s/%s" % (self.data, "groups.xml"), self)
                 core.fam.AddMonitor("%s/%s" % (self.data, "clients.xml"), self)
             except:
+                print("Unable to add file monitor for groups.xml or clients.xml")
                 raise Bcfg2.Server.Plugin.PluginInitError
         self.states = {}
         if watch_clients:
@@ -102,12 +112,13 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
                                    lambda:self.clients.keys(),
                                    self.get_client_names_by_groups,
                                    self.get_client_names_by_profiles,
-                                   self.get_all_group_names)
+                                   self.get_all_group_names,
+                                   self.get_all_groups_in_category)
 
     @classmethod
     def init_repo(cls, repo, groups, os_selection, clients):
         path = '%s/%s' % (repo, cls.name)
-        cls.make_path(path)
+        os.makedirs(path)
         open("%s/Metadata/groups.xml" %
              repo, "w").write(groups % os_selection)
         open("%s/Metadata/clients.xml" %
@@ -120,7 +131,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         return root
 
     def search_group(self, group_name, tree):
-        '''find a group'''
+        """Find a group."""
         for node in tree.findall("//Group"):
             if node.get("name") == group_name:
                 return node
@@ -130,7 +141,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         return None
 
     def add_group(self, group_name, attribs):
-        '''add group to groups.xml'''
+        """Add group to groups.xml."""
         tree = lxml.etree.parse(self.data + "/groups.xml")
         root = tree.getroot()
         element = lxml.etree.Element("Group", name=group_name)
@@ -155,7 +166,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         group_tree.close()
 
     def update_group(self, group_name, attribs):
-        '''Update a groups attributes'''
+        """Update a groups attributes."""
         tree = lxml.etree.parse(self.data + "/groups.xml")
         root = tree.getroot()
         node = self.search_group(group_name, tree)
@@ -177,7 +188,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         group_tree.close()
 
     def remove_group(self, group_name):
-        '''Remove a group'''
+        """Remove a group."""
         tree = lxml.etree.parse(self.data + "/groups.xml")
         root = tree.getroot()
         node = self.search_group(group_name, tree)
@@ -199,7 +210,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         group_tree.close()
 
     def add_bundle(self, bundle_name):
-        '''add bundle to groups.xml'''
+        """Add bundle to groups.xml."""
         tree = lxml.etree.parse(self.data + "/groups.xml")
         root = tree.getroot()
         element = lxml.etree.Element("Bundle", name=bundle_name)
@@ -222,7 +233,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         group_tree.close()
 
     def remove_bundle(self, bundle_name):
-        '''Remove a bundle'''
+        """Remove a bundle."""
         tree = lxml.etree.parse(self.data + "/groups.xml")
         root = tree.getroot()
         node = self.search_group(bundle_name, tree)
@@ -244,7 +255,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         group_tree.close()
 
     def search_client(self, client_name, tree):
-        '''find a client'''
+        """Find a client."""
         for node in tree.findall("//Client"):
             if node.get("name") == client_name:
                 return node
@@ -254,7 +265,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         return None
 
     def add_client(self, client_name, attribs):
-        '''add client to clients.xml'''
+        """Add client to clients.xml."""
         tree = lxml.etree.parse(self.data + "/clients.xml")
         root = tree.getroot()
         element = lxml.etree.Element("Client", name=client_name)
@@ -279,7 +290,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         client_tree.close()
 
     def update_client(self, client_name, attribs):
-        '''Update a clients attributes'''
+        """Update a clients attributes."""
         tree = lxml.etree.parse(self.data + "/clients.xml")
         root = tree.getroot()
         node = self.search_client(client_name, tree)
@@ -301,7 +312,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         client_tree.close()
 
     def HandleEvent(self, event):
-        '''Handle update events for data files'''
+        """Handle update events for data files."""
         filename = event.filename.split('/')[-1]
         if filename in ['groups.xml', 'clients.xml']:
             dest = filename
@@ -442,7 +453,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
                     del self.bad_clients[bclient]
 
     def set_profile(self, client, profile, addresspair):
-        '''Set group parameter for provided client'''
+        """Set group parameter for provided client."""
         self.logger.info("Asserting client %s profile to %s" % (client, profile))
         if False in self.states.values():
             raise MetadataRuntimeError
@@ -471,7 +482,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         self.write_back_clients()
 
     def write_back_clients(self):
-        '''Write changes to client.xml back to disk'''
+        """Write changes to client.xml back to disk."""
         try:
             datafile = open("%s/%s" % (self.data, 'clients.xml.new'), 'w')
         except IOError:
@@ -488,9 +499,6 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             item.tail = None
             item.text = None
         newcontents = lxml.etree.tostring(dataroot, pretty_print=True)
-        if len(newcontents) == 0:
-            self.logger.error("Metadata: lxml returns 0 length string")
-            raise MetadataRuntimeError
 
         fd = datafile.fileno()
         while self.locked(fd) == True:
@@ -502,11 +510,15 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             self.logger.error("Metadata: Failed to write new clients data to clients.xml.new", exc_info=1)
             os.unlink("%s/%s" % (self.data, "clients.xml.new"))
             raise MetadataRuntimeError
-        datafile.close()            
+        datafile.close()
+
+        # check if clients.xml is a symlink
+        clientsxml = "%s/%s" % (self.data, 'clients.xml')
+        if os.path.islink(clientsxml):
+            clientsxml = os.readlink(clientsxml)
 
         try:
-            os.rename("%s/%s" % (self.data, 'clients.xml.new'),
-                      "%s/%s" % (self.data, 'clients.xml'))
+            os.rename("%s/%s" % (self.data, 'clients.xml.new'), clientsxml)
         except:
             self.logger.error("Metadata: Failed to rename clients.xml.new")
             raise MetadataRuntimeError
@@ -519,7 +531,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         return False
 
     def resolve_client(self, addresspair):
-        '''Lookup address locally or in DNS to get a hostname'''
+        """Lookup address locally or in DNS to get a hostname."""
         if addresspair in self.session_cache:
             (stamp, uuid) = self.session_cache[addresspair]
             if time.time() - stamp < 90:
@@ -541,7 +553,9 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             raise MetadataConsistencyError
 
     def get_initial_metadata(self, client):
-        '''Return the metadata for a given client'''
+        """Return the metadata for a given client."""
+        if False in self.states.values():
+            raise MetadataRuntimeError
         client = client.lower()
         if client in self.aliases:
             client = self.aliases[client]
@@ -586,6 +600,12 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         [all_groups.update(g[1]) for g in self.groups.values()]
         return all_groups
 
+    def get_all_groups_in_category(self, category):
+        all_groups = set()
+        [all_groups.add(g) for g in self.categories \
+                if self.categories[g] == category]
+        return all_groups
+
     def get_client_names_by_profiles(self, profiles):
         return [client for client, profile in self.clients.iteritems() \
                 if profile in profiles]
@@ -620,7 +640,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             imd.connectors.append(source)
 
     def validate_client_address(self, client, addresspair):
-        '''Check address against client'''
+        """Check address against client."""
         address = addresspair[0]
         if client in self.floating:
             self.debug_log("Client %s is floating" % client)
@@ -643,7 +663,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             return False
 
     def AuthenticateConnection(self, cert, user, password, address):
-        '''This function checks auth creds'''
+        """This function checks auth creds."""
         if cert:
             id_method = 'cert'
             certinfo = dict([x[0] for x in cert['subject']])
@@ -711,7 +731,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
         return True
 
     def process_statistics(self, meta, _):
-        '''Hook into statistics interface to toggle clients in bootstrap mode'''
+        """Hook into statistics interface to toggle clients in bootstrap mode."""
         client = meta.hostname
         if client in self.auth and self.auth[client] == 'bootstrap':
             self.logger.info("Asserting client %s auth mode to cert" % client)
@@ -721,8 +741,12 @@ class Metadata(Bcfg2.Server.Plugin.Plugin,
             self.write_back_clients()
 
     def viz(self, hosts, bundles, key, colors):
-        '''admin mode viz support'''
+        """Admin mode viz support."""
         groups_tree = lxml.etree.parse(self.data + "/groups.xml")
+        try:
+            groups_tree.xinclude()
+        except lxml.etree.XincludeError:
+            self.logger.error("Failed to process XInclude for file %s" % dest)
         groups = groups_tree.getroot()
         categories = {'default':'grey83'}
         instances = {}
