@@ -1,12 +1,20 @@
 import cPickle
 import copy
 import gzip
+import glob
 import logging
 import lxml.etree
 import os
 import re
+import sys
 import urllib2
 
+# FIXME: Remove when server python dep is 2.5 or greater
+if sys.version_info >= (2, 5):
+    from hashlib import md5
+else:
+    from md5 import md5
+
 import Bcfg2.Logger
 import Bcfg2.Server.Plugin
 
@@ -24,7 +32,8 @@ logger = logging.getLogger('Packages')
 def source_from_xml(xsource):
     ret = dict([('rawurl', False), ('url', False)])
     for key, tag in [('groups', 'Group'), ('components', 'Component'),
-                     ('arches', 'Arch')]:
+                     ('arches', 'Arch'), ('blacklist', 'Blacklist'),
+                     ('whitelist', 'Whitelist')]:
         ret[key] = [item.text for item in xsource.findall(tag)]
     # version and component need to both contain data for sources to work
     try:
@@ -33,6 +42,13 @@ def source_from_xml(xsource):
         ret['version'] = 'placeholder'
     if ret['components'] == []:
         ret['components'] = ['placeholder']
+    try:
+        if xsource.find('Recommended').text in ['True', 'true']:
+            ret['recommended'] = True
+        else:
+            ret['recommended'] = False
+    except:
+        ret['recommended'] = False
     if xsource.find('RawURL') is not None:
         ret['rawurl'] = xsource.find('RawURL').text
         if not ret['rawurl'].endswith('/'):
@@ -43,9 +59,24 @@ def source_from_xml(xsource):
             ret['url'] += '/'
     return ret
 
+def _fetch_url(url):
+    if '@' in url:
+        mobj = re.match('(\w+://)([^:]+):([^@]+)@(.*)$', url)
+        if not mobj:
+            raise ValueError
+        user = mobj.group(2)
+        passwd = mobj.group(3)
+        url = mobj.group(1) + mobj.group(4)
+        auth = urllib2.HTTPBasicAuthHandler(urllib2.HTTPPasswordMgrWithDefaultRealm())
+        auth.add_password(None, url, user, passwd)
+        urllib2.install_opener(urllib2.build_opener(auth))
+    return urllib2.urlopen(url).read()
+
 class Source(object):
     basegroups = []
-    def __init__(self, basepath, url, version, arches, components, groups, rawurl):
+
+    def __init__(self, basepath, url, version, arches, components, groups, rawurl,
+                 blacklist, whitelist, recommended):
         self.basepath = basepath
         self.version = version
         self.components = components
@@ -55,12 +86,18 @@ class Source(object):
         self.arches = arches
         self.deps = dict()
         self.provides = dict()
-        self.cachefile = None
+        self.blacklist = set(blacklist)
+        self.whitelist = set(whitelist)
+        self.cachefile = '%s/cache-%s' % (self.basepath, md5(cPickle.dumps( \
+                [self.version, self.components, self.url, \
+                self.rawurl, self.groups, self.arches])).hexdigest())
+        self.recommended = recommended
+        self.url_map = []
 
     def load_state(self):
         pass
 
-    def setup_data(self):
+    def setup_data(self, force_update=False):
         should_read = True
         should_download = False
         if os.path.exists(self.cachefile):
@@ -77,9 +114,12 @@ class Source(object):
                 logger.error("Packages: File read failed; falling back to file download")
                 should_download = True
 
-        if should_download:
-            self.update()
-            self.read_files()
+        if should_download or force_update:
+            try:
+                self.update()
+                self.read_files()
+            except:
+                logger.error("Failed to update source", exc_info=1)
 
     def get_urls(self):
         return []
@@ -89,6 +129,17 @@ class Source(object):
         return [self.escape_url(url) for url in self.urls]
     files = property(get_files)
 
+    def get_vpkgs(self, meta):
+        agroups = ['global'] + [a for a in self.arches if a in meta.groups]
+        vdict = dict()
+        for agrp in agroups:
+            for key, value in self.provides[agrp].iteritems():
+                if key not in vdict:
+                    vdict[key] = set(value)
+                else:
+                    vdict[key].update(value)
+        return vdict
+
     def escape_url(self, url):
         return "%s/%s" % (self.basepath, url.replace('/', '@'))
 
@@ -103,7 +154,10 @@ class Source(object):
             logger.info("Packages: Updating %s" % url)
             fname = self.escape_url(url)
             try:
-                data = urllib2.urlopen(url).read()
+                data = _fetch_url(url)
+            except ValueError:
+                logger.error("Packages: Bad url string %s" % url)
+                continue
             except urllib2.HTTPError, h:
                 logger.error("Packages: Failed to fetch url %s. code=%s" \
                              % (url, h.code))
@@ -133,110 +187,26 @@ class Source(object):
     def is_package(self, metadata, _):
         return False
 
-    def resolve_requirement(self, metadata, requirement, packages, debug=False):
-        '''Resolve requirement to packages and or additional requirements
-
-        Arguments:
-        metadata -- client metadata instance
-        requirement -- name of requirement
-        debug -- boolean debug flag
-
-        Returns => (packages, unresolved requirements)
-        '''
-        if self.is_package(metadata, requirement):
-            item_is_pkg = True
-        else:
-            item_is_pkg = False
-
-        try:
-            provset = self.get_provides(metadata, requirement)
-            item_is_virt = True
-        except:
-            item_is_virt = False
-
-        if True not in [item_is_pkg, item_is_virt]:
-            raise NoData
-
-        if debug:
-            logger.debug("Handling requirement %s" % (requirement))
-
-        if item_is_pkg and not item_is_virt:
-            deps = set()
-            if debug:
-                logger.debug("Adding Package %s" % requirement)
-            try:
-                deps = self.get_deps(metadata, requirement)
-                if debug:
-                    logger.debug("Package %s: adding new deps %s" \
-                                 % (requirement, deps))
-            except:
-                pass
-            return (set([requirement]), deps)
-        if item_is_virt:
-            if item_is_pkg:
-                # requirement can be used to satisfy virt requirement
-                provset.add(requirement)
-            if debug:
-                logger.debug("Requirement %s provided by %s" \
-                             % (requirement, provset))
-
-            if len(provset) == 1:
-                # single choice for requirement
-                deps = set()
-                pname = list(provset)[0]
-                if debug:
-                    logger.debug("Adding Package %s for %s" % (pname, requirement))
-                try:
-                    deps = self.get_deps(metadata, pname)
-                    if debug:
-                        logger.debug("Package %s: adding new deps %s" \
-                                     % (pname, deps))
-                except:
-                    pass
-                return (set([pname]), deps)
-
-            satisfiers = provset.intersection(packages)
-            if satisfiers:
-                # requirement already satisfied
-                if debug:
-                    logger.debug("Requirement %s satisfied by %s" \
-                                 % (requirement, satisfiers))
-                return (set(), set())
-
-            if item_is_pkg:
-                # fall back to package with name of requirement
-                deps = set()
-                if debug:
-                    logger.debug("Adding Package %s for %s" \
-                                 % (requirement, requirement))
-                try:
-                    deps = self.get_deps(metadata, requirement)
-                    if debug:
-                        logger.debug("Package %s: adding new deps %s" \
-                                     % (requirement, deps))
-                except:
-                    pass
-                return (set([requirement]), deps)
-
-            # choice data is here, but not forced by currently resolved requirements
-            raise SomeData
+    def get_url_info(self):
+        return {'groups': copy.copy(self.groups), \
+            'urls': [copy.deepcopy(url) for url in self.url_map]}
 
 class YUMSource(Source):
     xp = '{http://linux.duke.edu/metadata/common}'
     rp = '{http://linux.duke.edu/metadata/rpm}'
     rpo = '{http://linux.duke.edu/metadata/repo}'
     fl = '{http://linux.duke.edu/metadata/filelists}'
-    basegroups = ['redhat', 'centos']
+    basegroups = ['yum', 'redhat', 'centos', 'fedora']
     ptype = 'yum'
 
-    def __init__(self, basepath, url, version, arches, components, groups, rawurl):
+    def __init__(self, basepath, url, version, arches, components, groups,
+                 rawurl, blacklist, whitelist, recommended):
         Source.__init__(self, basepath, url, version, arches, components,
-                        groups, rawurl)
+                        groups, rawurl, blacklist, whitelist, recommended)
         if not self.rawurl:
             self.baseurl = self.url + '%(version)s/%(component)s/%(arch)s/'
         else:
             self.baseurl = self.rawurl
-        self.cachefile = self.escape_url(self.baseurl) + '.data'
         self.packages = dict()
         self.deps = dict([('global', dict())])
         self.provides = dict([('global', dict())])
@@ -247,20 +217,25 @@ class YUMSource(Source):
     def save_state(self):
         cache = file(self.cachefile, 'wb')
         cPickle.dump((self.packages, self.deps, self.provides,
-                      self.filemap), cache, 2)
+                      self.filemap, self.url_map), cache, 2)
         cache.close()
 
     def load_state(self):
         data = file(self.cachefile)
         (self.packages, self.deps, self.provides, \
-         self.filemap) = cPickle.load(data)
+         self.filemap, self.url_map) = cPickle.load(data)
 
     def get_urls(self):
         surls = list()
+        self.url_map = []
         for arch in self.arches:
             usettings = [{'version': self.version, 'component':comp,
                           'arch':arch} for comp in self.components]
-            surls.append((arch, [self.baseurl % setting for setting in usettings]))
+            for setting in usettings:
+                setting['groups'] = self.groups
+                setting['url'] = self.baseurl % setting
+                self.url_map.append(copy.deepcopy(setting))
+            surls.append((arch, [setting['url'] for setting in usettings]))
         urls = []
         for (sarch, surl_list) in surls:
             for surl in surl_list:
@@ -268,8 +243,15 @@ class YUMSource(Source):
                     surl += '/'
                 rmdurl = surl + 'repodata/repomd.xml'
                 try:
-                    repomd = urllib2.urlopen(rmdurl).read()
+                    repomd = _fetch_url(rmdurl)
                     xdata = lxml.etree.XML(repomd)
+                except ValueError:
+                    logger.error("Packages: Bad url string %s" % rmdurl)
+                    continue
+                except urllib2.HTTPError, h:
+                    logger.error("Packages: Failed to fetch url %s. code=%s" \
+                             % (rmdurl, h.code))
+                    continue
                 except:
                     logger.error("Failed to process url %s" % rmdurl)
                     continue
@@ -347,41 +329,35 @@ class YUMSource(Source):
         arch = [a for a in self.arches if a in metadata.groups]
         if not arch:
             return False
-        return item in self.packages['global'] or item in self.packages[arch[0]]
+        return (item in self.packages['global'] or item in self.packages[arch[0]]) and \
+               item not in self.blacklist and \
+               ((len(self.whitelist) == 0) or item in self.whitelist)
 
-    def get_provides(self, metadata, required):
-        ret = set()
-        arches = [a for a in self.arches if a in metadata.groups]
-        if not arches:
-            raise NoData
-        if required in self.provides['global']:
-            ret.update(Source.get_provides(self, metadata, required))
-        elif required in self.provides[arches[0]]:
-            ret.update(Source.get_provides(self, metadata, required))
-        else:
-            for arch in ['global'] + arches:
-                if required in self.filemap[arch]:
-                    ret.update(self.filemap[arch][required])
-        if ret:
-            return ret
-        else:
-            raise NoData
+    def get_vpkgs(self, metadata):
+        rv = Source.get_vpkgs(self, metadata)
+        for arch, fmdata in self.filemap.iteritems():
+            if arch not in metadata.groups and arch != 'global':
+                continue
+            for filename, pkgs in fmdata.iteritems():
+                rv[filename] = pkgs
+        return rv
 
-    def resolve_requirement(self, m, r, p, d):
-        if r.startswith('rpmlib'):
-            return (set(), set())
-        else:
-            return Source.resolve_requirement(self, m, r, p, d)
+    def filter_unknown(self, unknown):
+        filtered = set([u for u in unknown if u.startswith('rpmlib')])
+        unknown.difference_update(filtered)
 
 class APTSource(Source):
-    basegroups = ['debian', 'ubuntu', 'nexenta']
+    basegroups = ['apt', 'debian', 'ubuntu', 'nexenta']
     ptype = 'deb'
 
-    def __init__(self, basepath, url, version, arches, components, groups, rawurl):
-        Source.__init__(self, basepath, url, version, arches, components, groups, rawurl)
-        self.cachefile = self.escape_url(self.url) + '.data'
+    def __init__(self, basepath, url, version, arches, components, groups,
+                 rawurl, blacklist, whitelist, recommended):
+        Source.__init__(self, basepath, url, version, arches, components, groups,
+                        rawurl, blacklist, whitelist, recommended)
         self.pkgnames = set()
-        print self.url
+
+        self.url_map = [{'rawurl': self.rawurl, 'url': self.url, 'version': self.version, \
+            'components': self.components, 'arches': self.arches, 'groups': self.groups}]
 
     def save_state(self):
         cache = file(self.cachefile, 'wb')
@@ -393,23 +369,33 @@ class APTSource(Source):
         data = file(self.cachefile)
         self.pkgnames, self.deps, self.provides = cPickle.load(data)
 
-    def get_urls(self):
-        return ["%sdists/%s/%s/binary-%s/Packages.gz" % \
-                (self.url, self.version, part, arch) for part in self.components \
-                for arch in self.arches]
-    urls = property(get_urls)
+    def filter_unknown(self, unknown):
+        filtered = set([u for u in unknown if u.startswith('choice')])
+        unknown.difference_update(filtered)
 
-    def get_aptsrc(self):
-        return ["deb %s %s %s" % (self.url, self.version,
-                                  " ".join(self.components)),
-                "deb-src %s %s %s" % (self.url, self.version,
-                                      " ".join(self.components))]
+    def get_urls(self):
+        if not self.rawurl:
+            return ["%sdists/%s/%s/binary-%s/Packages.gz" % \
+                    (self.url, self.version, part, arch) for part in self.components \
+                    for arch in self.arches]
+        else:
+            return ["%sPackages.gz" % (self.rawurl)]
+    urls = property(get_urls)
 
     def read_files(self):
         bdeps = dict()
         bprov = dict()
+        if self.recommended:
+            depfnames = ['Depends', 'Pre-Depends', 'Recommends']
+        else:
+            depfnames = ['Depends', 'Pre-Depends']            
         for fname in self.files:
-            barch = [x for x in fname.split('@') if x.startswith('binary-')][0][7:]
+            if not self.rawurl:
+                barch = [x for x in fname.split('@') if x.startswith('binary-')][0][7:]
+            else:
+                # RawURL entries assume that they only have one <Arch></Arch>
+                # element and that it is the architecture of the source.
+                barch = self.arches[0]
             if barch not in bdeps:
                 bdeps[barch] = dict()
                 bprov[barch] = dict()
@@ -423,12 +409,12 @@ class APTSource(Source):
                 if words[0] == 'Package':
                     pkgname = words[1].strip().rstrip()
                     self.pkgnames.add(pkgname)
-                elif words[0] == 'Depends':
                     bdeps[barch][pkgname] = []
+                elif words[0] in depfnames:
                     vindex = 0
                     for dep in words[1].split(','):
                         if '|' in dep:
-                            cdeps = [re.sub('\(.*\)', '', cdep) for cdep in dep.split('|')]
+                            cdeps = [re.sub('\s+', '', re.sub('\(.*\)', '', cdep)) for cdep in dep.split('|')]
                             dyn_dname = "choice-%s-%s-%s" % (pkgname, barch, vindex)
                             vindex += 1
                             bdeps[barch][pkgname].append(dyn_dname)
@@ -477,46 +463,54 @@ class APTSource(Source):
         self.save_state()
 
     def is_package(self, _, pkg):
-        return pkg in self.pkgnames
-
-    def get_provides(self, metadata, pkgname):
-        arches = [ar for ar in self.provides if ar in metadata.groups]
-        for arch in ['global'] + arches:
-            if pkgname in self.provides[arch]:
-                return set(self.provides[arch][pkgname])
-        raise NoData
+        return pkg in self.pkgnames and \
+               pkg not in self.blacklist and \
+               (len(self.whitelist) == 0 or pkg in self.whitelist)
 
 class Packages(Bcfg2.Server.Plugin.Plugin,
                Bcfg2.Server.Plugin.StructureValidator,
-               Bcfg2.Server.Plugin.Generator):
+               Bcfg2.Server.Plugin.Generator,
+               Bcfg2.Server.Plugin.Connector):
     name = 'Packages'
+    conflicts = ['Pkgmgr']
     experimental = True
-    __rmi__ = ['Refresh']
+    __rmi__ = Bcfg2.Server.Plugin.Plugin.__rmi__ + ['Refresh', 'Reload']
 
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
         Bcfg2.Server.Plugin.StructureValidator.__init__(self)
         Bcfg2.Server.Plugin.Generator.__init__(self)
+        Bcfg2.Server.Plugin.Connector.__init__(self)
         self.cachepath = self.data + '/cache'
+        self.sentinels = set()
+        self.sources = []
+        self.disableResolver = False
+        self.disableMetaData = False
+        self.virt_pkgs = dict()
 
         if not os.path.exists(self.cachepath):
             # create cache directory if needed
             os.makedirs(self.cachepath)
-        try:
-            xdata = lxml.etree.parse(self.data + '/config.xml').getroot()
-        except IOError:
-            self.logger.error("Failed to read Packages configuration. Have"
-                              " you created your config.xml file?")
-            raise Bcfg2.Server.Plugin.PluginInitError
-        self.sentinels = set()
-        self.sources = []
-        for s in xdata.findall('APTSource'):
-            self.sources.append(APTSource(self.cachepath, **source_from_xml(s)))
-        for s in xdata.findall('YUMSource'):
-            self.sources.append(YUMSource(self.cachepath, **source_from_xml(s)))
-        for source in self.sources:
-            source.setup_data()
-            self.sentinels.update(source.basegroups)
+        self._load_config()
+
+    def get_relevant_groups(self, meta):
+        mgrps = list(set([g for g in meta.groups for s in self.get_matching_sources(meta) \
+                          if g in s.basegroups or g in s.groups or g in s.arches]))
+        mgrps.sort()
+        return tuple(mgrps)
+
+    def build_vpkgs_entry(self, meta):
+        # build single entry for all matching sources
+        mgrps = self.get_relevant_groups(meta)
+        vpkgs = dict()
+        for source in self.get_matching_sources(meta):
+            s_vpkgs = source.get_vpkgs(meta)
+            for name, prov_set in s_vpkgs.iteritems():
+                if name not in vpkgs:
+                    vpkgs[name] = set(prov_set)
+                else:
+                    vpkgs[name].update(prov_set)
+        return vpkgs
 
     def get_matching_sources(self, meta):
         return [s for s in self.sources if s.applies(meta)]
@@ -533,13 +527,14 @@ class Packages(Bcfg2.Server.Plugin.Plugin,
             if [x for x in metadata.groups if x in source.basegroups]:
                 entry.set('type', source.ptype)
 
-    def complete(self, meta, packages, debug=False):
+    def complete(self, meta, input_requirements, debug=False):
         '''Build the transitive closure of all package dependencies
 
         Arguments:
         meta - client metadata instance
         packages - set of package names
         debug - print out debug information for the decision making process
+        returns => (set(packages), set(unsatisfied requirements), package type)
         '''
         sources = self.get_matching_sources(meta)
         # reverse list so that priorities correspond to file order
@@ -550,46 +545,104 @@ class Packages(Bcfg2.Server.Plugin.Plugin,
         ptype = set([s.ptype for s in sources])
         if len(ptype) < 1:
             return set(), set(), 'failed'
-        pkgs = set(packages)
-        needed = set(packages)
-        unknown = set()
+
+        # setup vpkg cache
+        pgrps = self.get_relevant_groups(meta)
+        if pgrps not in self.virt_pkgs:
+            self.virt_pkgs[pgrps] = self.build_vpkgs_entry(meta)
+        vpkg_cache = self.virt_pkgs[pgrps]
+
+        # unclassified is set of unsatisfied requirements (may be pkg for vpkg)
+        unclassified = set(input_requirements)
+        vpkgs = set()
+        both = set()
+        pkgs = set(input_requirements)
+
+        packages = set()
         examined = set()
+        unknown = set()
 
-        final_pass = set()
-        in_final_pass = False
+        final_pass = False
+        really_done = False
+        # do while unclassified or vpkgs or both or pkgs
+        while unclassified or pkgs or both or final_pass:
+            #print len(unclassified), len(pkgs), len(both), len(vpkgs), final_pass
+            if really_done: 
+                break
+            if len(unclassified) + len(pkgs) + len(both) == 0:
+                # one more pass then exit
+                really_done = True
 
-        while needed:
-            # process requirements until all done or no progress
-            current = needed.pop()
-            examined.add(current)
-            found = False
-            for source in sources:
-                try:
-                    newp, newr = source.resolve_requirement(meta, current,
-                                                            packages, debug)
-                    found = True
-                    break
-                except NoData:
-                    continue
-                except SomeData:
-                    if not in_final_pass:
-                        final_pass.add(current)
-                except:
-                    self.logger.error("Packages: resolve_requirement call failed unexpectedly", exc_info=1)
-            if found:
-                in_final_pass = False
-                pkgs = pkgs.union(newp)
-                needed = needed.union(newr)
-                needed.difference_update(examined)
+            while unclassified:
+                current = unclassified.pop()
+                examined.add(current)
+                is_pkg = True in [source.is_package(meta, current) for source in sources]
+                is_vpkg = current in vpkg_cache
+
+                if is_pkg and is_vpkg:
+                    both.add(current)
+                elif is_pkg and not is_vpkg:
+                    pkgs.add(current)
+                elif is_vpkg and not is_pkg:
+                    vpkgs.add(current)
+                elif not is_vpkg and not is_pkg:
+                    unknown.add(current)
+
+            while pkgs:
+                # direct packages; current can be added, and all deps should be resolved
+                current = pkgs.pop()
+                if debug:
+                    self.logger.debug("Packages: handling package requirement %s" % (current))
+                deps = ()
+                for source in sources:
+                    if source.is_package(meta, current):
+                        try:
+                            deps = source.get_deps(meta, current)
+                            break
+                        except:
+                            continue
+                packages.add(current)
+                newdeps = set(deps).difference(examined)
+                if debug and newdeps:
+                    self.logger.debug("Packages: Package %s added requirements %s" % (current, newdeps))
+                unclassified.update(newdeps)
+
+            satisfied_vpkgs = set()
+            for current in vpkgs:
+                # virtual dependencies, satisfied if one of N in the config, or can be forced if only one provider
+                if len(vpkg_cache[current]) == 1:
+                    if debug:
+                        self.logger.debug("Packages: requirement %s satisfied by %s" % (current, vpkg_cache[current]))
+                    unclassified.update(vpkg_cache[current].difference(examined))
+                    satisfied_vpkgs.add(current)
+                elif [item for item in vpkg_cache[current] if item in packages]:
+                    if debug:
+                        self.logger.debug("Packages: requirement %s satisfied by %s" % (current, [item for item in vpkg_cache[current] if item in packages]))
+                    satisfied_vpkgs.add(current)
+            vpkgs.difference_update(satisfied_vpkgs)
+
+            satisfied_both = set()
+            for current in both:
+                # packages that are both have virtual providers as well as a package with that name
+                # allow use of virt through explicit specification, then fall back to forcing current on last pass
+                if [item for item in vpkg_cache[current] if item in packages]:
+                    if debug:
+                        self.logger.debug("Packages: requirement %s satisfied by %s" % (current, [item for item in vpkg_cache[current] if item in packages]))
+                    satisfied_both.add(current)
+                elif current in input_requirements or final_pass:
+                    pkgs.add(current)
+                    satisfied_both.add(current)
+            both.difference_update(satisfied_both)
+
+            if len(unclassified) + len(pkgs) == 0:
+                final_pass = True
             else:
-                unknown.add(current)
+                final_pass = False
 
-            if not needed:
-                in_final_pass = True
-                needed.update(final_pass)
-                final_pass = set()
+            for source in sources:
+                source.filter_unknown(unknown)
 
-        return pkgs, unknown, ptype.pop()
+        return packages, unknown, ptype.pop()
 
     def validate_structures(self, meta, structures):
         '''Ensure client configurations include all needed prerequisites
@@ -598,14 +651,17 @@ class Packages(Bcfg2.Server.Plugin.Plugin,
         meta - client metadata instance
         structures - a list of structure-stage entry combinations
         '''
+        if self.disableResolver: return # Config requests no resolver
+
         initial = set([pkg.get('name') for struct in structures \
-                       for pkg in struct.findall('Package')])
+                       for pkg in struct.findall('Package') +
+                       struct.findall('BoundPackage')])
         news = lxml.etree.Element('Independent')
-        packages, unknown, ptype = self.complete(meta, initial)
-        logged_unknown = [x for x in unknown if not x.startswith('choice')]
-        if logged_unknown:
+        packages, unknown, ptype = self.complete(meta, initial,
+                                                 debug=self.debug_flag)
+        if unknown:
             self.logger.info("Got unknown entries")
-            self.logger.info(logged_unknown)
+            self.logger.info(list(unknown))
         newpkgs = list(packages.difference(initial))
         newpkgs.sort()
         for pkg in newpkgs:
@@ -628,41 +684,74 @@ class Packages(Bcfg2.Server.Plugin.Plugin,
         sources = self.get_matching_sources(meta)
         for source in sources:
             for pkgname in pkgnames:
-                try:
-                    deps = source.get_deps(meta, pkgname)
-                except:
-                    continue
-                for rpkg in deps:
-                    if rpkg in pkgnames:
-                        redundant.add(rpkg)
+                if source.is_pkg(meta, current):
+                    try:
+                        deps = source.get_deps(meta, pkgname)
+                    except:
+                        continue
+                    for rpkg in deps:
+                        if rpkg in pkgnames:
+                            redundant.add(rpkg)
         return pkgnames.difference(redundant), redundant
 
     def Refresh(self):
+        '''Packages.Refresh() => True|False\nReload configuration specification and download sources\n'''
+        self._load_config(force_update=True)
+        return True
+
+    def Reload(self):
         '''Packages.Refresh() => True|False\nReload configuration specification and sources\n'''
+        self._load_config()
+        return True
+
+    def _load_config(self, force_update=False):
+        '''
+        Load the configuration data and setup sources
+
+        Keyword args:
+            force_update    Force downloading repo data
+        '''
+        self.virt_pkgs = dict()
         try:
-            xdata = lxml.etree.parse(self.data + '/config.xml').getroot()
+            xdata = lxml.etree.parse(self.data + '/config.xml')
+            xdata.xinclude()
+            xdata = xdata.getroot()
+        except (lxml.etree.XIncludeError, \
+                lxml.etree.XMLSyntaxError), xmlerr:
+            self.logger.error("Package: Error processing xml: %s" % xmlerr)
+            raise Bcfg2.Server.Plugin.PluginInitError
         except IOError:
             self.logger.error("Failed to read Packages configuration. Have" +
                               " you created your config.xml file?")
             raise Bcfg2.Server.Plugin.PluginInitError
+
+        # Load Packages config
+        config = xdata.xpath('//Sources/Config')
+        if config:
+            if config[0].get("resolver", "enabled").lower() == "disabled":
+                self.logger.info("Packages: Resolver disabled")
+                self.disableResolver = True
+            if config[0].get("metadata", "enabled").lower() == "disabled":
+                self.logger.info("Packages: Metadata disabled")
+                self.disableResolver = True
+                self.disableMetaData = True
+
         self.sentinels = set()
         self.sources = []
-        for s in xdata.findall('APTSource'):
+        for s in xdata.findall('.//APTSource'):
             self.sources.append(APTSource(self.cachepath, **source_from_xml(s)))
-        for s in xdata.findall('YUMSource'):
+        for s in xdata.findall('.//YUMSource'):
             self.sources.append(YUMSource(self.cachepath, **source_from_xml(s)))
+        cachefiles = []
         for source in self.sources:
-            source.setup_data()
+            cachefiles.append(source.cachefile)
+            if not self.disableMetaData: source.setup_data(force_update)
             self.sentinels.update(source.basegroups)
-        for source in self.sources:
-            try:
-                source.update()
-            except:
-                self.logger.error("Failed to update source", exc_info=1)
-                continue
-            source.read_files()
-        return True
+        for cfile in glob.glob("%s/cache-*" % self.cachepath):
+            if cfile not in cachefiles:
+                os.unlink(cfile)
 
-if __name__ == '__main__':
-    Bcfg2.Logger.setup_logging('Packages', to_console=True)
-    aa = Packages(None, '/home/desai/tmp/bcfg2')
+    def get_additional_data(self, meta):
+        sdata = []
+        [sdata.extend(copy.deepcopy(src.url_map)) for src in self.get_matching_sources(meta)]
+        return dict(sources=sdata)
