@@ -1,5 +1,5 @@
-'''imports for Bcfg2.Server.Plugins'''
-__revision__ = '$Revision: 5476 $'
+"""Imports for Bcfg2.Server.Plugins."""
+__revision__ = '$Revision$'
 
 __all__ = [
            'Account',
@@ -7,10 +7,13 @@ __all__ = [
            'Bundler',
            'Bzr',
            'Cfg',
+           'Cvs',
+           'Darcs', 
            'Decisions',
            'Fossil',
            'Git',
            'GroupPatterns',
+           'Hg',
            'Hostbase',
            'Metadata',
            'NagiosGen',
