@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 '''Imports statistics.xml and clients.xml files in to database backend for new statistics engine'''
-__revision__ = '$Revision: 5027 $'
+__revision__ = '$Revision$'
 
 import os, sys, binascii
 try:
@@ -17,7 +17,7 @@ sys.path.pop()
 # Set DJANGO_SETTINGS_MODULE appropriately.
 os.environ['DJANGO_SETTINGS_MODULE'] = '%s.settings' % project_name
 
-from Bcfg2.Server.Reports.reports.models import Client, Interaction, Entries, Entries_interactions, Performance, Reason, Ping, TYPE_CHOICES, InternalDatabaseVersion
+from Bcfg2.Server.Reports.reports.models import *
 from lxml.etree import XML, XMLSyntaxError
 from getopt import getopt, GetoptError
 from datetime import datetime
@@ -25,12 +25,20 @@ from time import strptime
 from django.db import connection
 from Bcfg2.Server.Reports.updatefix import update_database
 import ConfigParser
-import difflib
+import logging
+import Bcfg2.Logger
+import platform
 
 def build_reason_kwargs(r_ent):
+    binary_file=False
     if r_ent.get('current_bfile', False):
-        contents = binascii.a2b_base64(r_ent.get('current_bfile'))
-        rc_diff = '\n'.join(difflib.ndiff([], contents.split('\n')))
+        binary_file=True
+        rc_diff = r_ent.get('current_bfile')
+        if len(rc_diff) > 1024*1024:
+            rc_diff = ''
+        elif len(rc_diff) == 0:
+            # No point in flagging binary if we have no data
+            binary_file=False
     elif r_ent.get('current_bdiff', False):
         rc_diff = binascii.a2b_base64(r_ent.get('current_bdiff'))
     elif r_ent.get('current_diff', False):
@@ -50,51 +58,38 @@ def build_reason_kwargs(r_ent):
                 version=r_ent.get('version', default=""),
                 current_version=r_ent.get('current_version', default=""),
                 current_exists=r_ent.get('current_exists', default="True").capitalize()=="True",
-                current_diff=rc_diff)
+                current_diff=rc_diff,
+                is_binary=binary_file)
 
 
-def load_stats(cdata, sdata, vlevel, quick=False, location=''):
-    cursor = connection.cursor()
+def load_stats(cdata, sdata, vlevel, logger, quick=False, location=''):
     clients = {}
-    cursor.execute("SELECT name, id from reports_client;")
-    [clients.__setitem__(a, b) for a, b in cursor.fetchall()]
-    
-    for node in sdata.findall('Node'):
-        name = node.get('name')
-        if not name in clients:
-            cursor.execute(\
-                "INSERT INTO reports_client VALUES (NULL, %s, %s, NULL, NULL)",
-                [datetime.now(), name])
-            clients[name] = cursor.lastrowid
-            if vlevel > 0:
-                print("Client %s added to db" % name)
-        else:
-            if vlevel > 0:
-                print("Client %s already exists in db" % name)
+    [clients.__setitem__(c.name, c) \
+        for c in Client.objects.all()]
 
     pingability = {}
     [pingability.__setitem__(n.get('name'), n.get('pingable', default='N')) \
-     for n in cdata.findall('Client')]
+        for n in cdata.findall('Client')]
 
     for node in sdata.findall('Node'):
         name = node.get('name')
-        c_inst = Client.objects.filter(id=clients[name])[0]
+        c_inst, created = Client.objects.get_or_create(name=name)
+        if vlevel > 0:
+           logger.info("Client %s added to db" % name)
+        clients[name] = c_inst
         try:
             pingability[name]
         except KeyError:
             pingability[name] = 'N'
         for statistics in node.findall('Statistics'):
-            t = strptime(statistics.get('time'))
-            # Maybe replace with django.core.db typecasts typecast_timestamp()?
-            # import from django.backends util
-            timestamp = datetime(t[0], t[1], t[2], t[3], t[4], t[5])
+            timestamp = datetime(*strptime(statistics.get('time'))[0:6])
             ilist = Interaction.objects.filter(client=c_inst,
                                                timestamp=timestamp)
             if ilist:
                 current_interaction = ilist[0]
                 if vlevel > 0:
-                    print("Interaction for %s at %s with id %s already exists"%(clients[name],
-                        datetime(t[0],t[1],t[2],t[3],t[4],t[5]),current_interaction.id))
+                    logger.info("Interaction for %s at %s with id %s already exists" % \
+                        (c_inst.id, timestamp, current_interaction.id))
                 continue
             else:
                 newint = Interaction(client=c_inst,
@@ -108,100 +103,85 @@ def load_stats(cdata, sdata, vlevel, quick=False, location=''):
                 newint.save()
                 current_interaction = newint
                 if vlevel > 0:
-                    print("Interaction for %s at %s with id %s INSERTED in to db"%(clients[name],
+                    logger.info("Interaction for %s at %s with id %s INSERTED in to db"%(c_inst.id,
                         timestamp, current_interaction.id))
 
 
+            counter_fields = { TYPE_CHOICES[0]: 0, TYPE_CHOICES[1]: 0, TYPE_CHOICES[2]: 0 }
             pattern = [('Bad/*', TYPE_CHOICES[0]),
                        ('Extra/*', TYPE_CHOICES[2]),
                        ('Modified/*', TYPE_CHOICES[1]),]
             for (xpath, type) in pattern:
                 for x in statistics.findall(xpath):
+                    counter_fields[type] = counter_fields[type] + 1
                     kargs = build_reason_kwargs(x)
-                    if not quick:
-                        rls = Reason.objects.filter(**kargs)
-                    else:
-                        rls = []
 
-                    if rls:
-                        rr = rls[0]
-                        if vlevel > 0:
-                            print "Reason exists: %s"% (rr.id)
-                    else:
-                        rr = Reason(**kargs)
+                    try:
+                        rr = None
+                        if not quick:
+                            try:
+                                rr = Reason.objects.filter(**kargs)[0]
+                            except IndexError:
+                                pass
+                        if not rr:
+                            rr = Reason(**kargs)
+                            rr.save()
+                            if vlevel > 0:
+                                logger.info("Created reason: %s" % rr.id)
+                    except Exception, ex:
+                        logger.error("Failed to create reason for %s: %s" % (x.get('name'), ex))
+                        rr = Reason(current_exists=x.get('current_exists',
+                                  default="True").capitalize()=="True")
                         rr.save()
-                        if vlevel > 0:
-                            print "Created reason: %s" % rr.id
 
-                    links = Entries.objects.filter(name=x.get('name'),
-                                               kind=x.tag)
+                    entry, created = Entries.objects.get_or_create(\
+                        name=x.get('name'), kind=x.tag)
                         
-                    if links:
-                        entry = links[0]
-                    else:
-                        entry = Entries(name=x.get('name'),
-                                      kind=x.tag)
-                        entry.save()
-
-                    interaction = Entries_interactions(entry=entry, reason=rr,
-                                                        interaction=current_interaction,
-                                                        type=type[0])
-                    interaction.save()
+                    Entries_interactions(entry=entry, reason=rr,
+                                         interaction=current_interaction,
+                                         type=type[0]).save()
                     if vlevel > 0:
-                        print "%s interaction created with reason id %s and entry %s" % (xpath, rr.id, entry.id)
+                        logger.info("%s interaction created with reason id %s and entry %s" % (xpath, rr.id, entry.id))
 
+            # Update interaction counters
+            current_interaction.bad_entries = counter_fields[TYPE_CHOICES[0]]
+            current_interaction.modified_entries = counter_fields[TYPE_CHOICES[1]]
+            current_interaction.extra_entries = counter_fields[TYPE_CHOICES[2]]
+            current_interaction.save()
+
+            mperfs = []
             for times in statistics.findall('OpStamps'):
                 for metric, value in times.items():
+                    mmatch = []
                     if not quick:
                         mmatch = Performance.objects.filter(metric=metric, value=value)
-                    else:
-                        mmatch = []
-                    
+
                     if mmatch:
-                        item_id = mmatch[0].id
+                        mperf = mmatch[0]
                     else:
                         mperf = Performance(metric=metric, value=value)
                         mperf.save()
-                        item_id = mperf.id
-                    try:
-                        cursor.execute("INSERT INTO reports_performance_interaction VALUES (NULL, %s, %s);",
-                                       [item_id, current_interaction.id])
-                    except:
-                        pass
-
-    if vlevel > 1:
-        print("----------------INTERACTIONS SYNCED----------------")
-    cursor.execute("select reports_interaction.id, x.client_id from (select client_id, MAX(timestamp) as timer from reports_interaction Group BY client_id) x, reports_interaction where reports_interaction.client_id = x.client_id AND reports_interaction.timestamp = x.timer")
-    for row in cursor.fetchall():
-        cursor.execute("UPDATE reports_client SET current_interaction_id = %s where reports_client.id = %s",
-                       [row[0],row[1]])
-    if vlevel > 1:
-        print("------------LATEST INTERACTION SET----------------")
+                    mperfs.append(mperf)
+            current_interaction.performance_items.add(*mperfs)
 
     for key in pingability.keys():
         if key not in clients:
-            #print "Ping Save Problem with client %s" % name
             continue
-        cmatch = Client.objects.filter(id=clients[key])[0]
-        pmatch = Ping.objects.filter(client=cmatch).order_by('-endtime')
-        if pmatch:
-            if pmatch[0].status == pingability[key]:
-                pmatch[0].endtime = datetime.now()
-                pmatch[0].save()
-            else:
-                newp = Ping(client=cmatch, status=pingability[key],
-                            starttime=datetime.now(),
-                            endtime=datetime.now())
-                newp.save()
-        else:
-            newp = Ping(client=cmatch, status=pingability[key],
-                        starttime=datetime.now(), endtime=datetime.now())
-            newp.save()
+        try:
+            pmatch = Ping.objects.filter(client=clients[key]).order_by('-endtime')[0]
+            if pmatch.status == pingability[key]:
+                pmatch.endtime = datetime.now()
+                pmatch.save()
+                continue
+        except IndexError:
+            pass
+        Ping(client=clients[key], status=pingability[key],
+             starttime=datetime.now(),
+             endtime=datetime.now()).save()
 
     if vlevel > 1:
-        print "---------------PINGDATA SYNCED---------------------"
+        logger.info("---------------PINGDATA SYNCED---------------------")
 
-    connection._commit()
     #Clients are consistent
 
 if __name__ == '__main__':
@@ -210,14 +190,15 @@ if __name__ == '__main__':
     cpath = "/etc/bcfg2.conf"
     clientpath = False
     statpath = False
+    syslog = False
     
     try:
-        opts, args = getopt(argv[1:], "hvudc:s:", ["help", "verbose", "updates" ,
+        opts, args = getopt(argv[1:], "hvudc:s:CS", ["help", "verbose", "updates" ,
                                                    "debug", "clients=", "stats=",
-                                                   "config="])
+                                                   "config=", "syslog"])
     except GetoptError, mesg:
         # print help information and exit:
-        print "%s\nUsage:\nimportscript.py [-h] [-v] [-u] [-d] [-C bcfg2 config file] [-c clients-file] [-s statistics-file]" % (mesg) 
+        print "%s\nUsage:\nimportscript.py [-h] [-v] [-u] [-d] [-S] [-C bcfg2 config file] [-c clients-file] [-s statistics-file]" % (mesg) 
         raise SystemExit, 2
 
     for o, a in opts:
@@ -230,6 +211,7 @@ if __name__ == '__main__':
             print "C : path to bcfg2.conf config file."
             print "c : clients.xml file"
             print "s : statistics.xml file"
+            print "S : syslog; output to syslog"
             raise SystemExit
         if o in ["-C", "--config"]:
             cpath = a
@@ -245,6 +227,14 @@ if __name__ == '__main__':
 
         if o in ("-s", "--stats"):
             statpath = a
+        if o in ("-S", "--syslog"):
+            syslog = True
+
+    logger = logging.getLogger('importscript.py')
+    logging.getLogger().setLevel(logging.INFO)
+    Bcfg2.Logger.setup_logging('importscript.py',
+                               True,
+                               syslog)
 
     cf = ConfigParser.ConfigParser()
     cf.read([cpath])
@@ -275,11 +265,6 @@ if __name__ == '__main__':
         raise SystemExit, 1
 
     q = '-O3' in sys.argv
-    try:
-        location = cf.get('components', 'bcfg2')
-    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
-        print "Could not read bcfg2 from bcfg2.conf; exiting"
-        raise SystemExit, 1
     # Be sure the database is ready for new schema
     update_database()
-    load_stats(clientsdata, statsdata, verb, quick=q, location=location)
+    load_stats(clientsdata, statsdata, verb, logger, quick=q, location=platform.node())
